/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package robocode.peer.robot;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import robocode.manager.NameManager;
import robocode.peer.TeamPeer;
import robocode.repository.FileSpecification;
import robocode.repository.RobotSpecification;
import robocode.security.RobocodeClassLoader;
import robocode.util.Utils;

public class RobotClassManager {

    private RobotSpecification robotSpecification = null;
    private Class robotClass = null;
    private Hashtable referencedClasses = null;
    private RobocodeClassLoader robotClassLoader = null;
    private robocode.control.RobotSpecification controlRobotSpecification =
        null;
    private String fullClassName = null;
    private TeamPeer teamManager = null;
    private String uid = null;

    public RobotClassManager(RobotSpecification robotSpecification) {
        this(robotSpecification, null);
    }

    public RobotClassManager(RobotSpecification robotSpecification,
                             TeamPeer teamManager) {
        referencedClasses = new Hashtable();
        robotClassLoader = null;
        controlRobotSpecification = null;
        fullClassName = null;
        this.teamManager = null;
        uid = "";
        this.robotSpecification = robotSpecification;
        fullClassName = robotSpecification.getName();
        this.teamManager = teamManager;
    }

    public String getRootPackage() {
        return getClassNameManager().getRootPackage();
    }

    public NameManager getClassNameManager() {
        return robotSpecification.getNameManager();
    }

    public void addReferencedClass(String className, String resolved) {
        referencedClasses.put(className, resolved);
    }

    public void addReferencedClasses(Vector v) {
        if (v == null)
            return;

        for (int i = 0; i < v.size(); i++) {
            String className = ((String) v.elementAt(i)).replace('/', '.');

            if ((getRootPackage() == null || className.indexOf("java") != 0 && className.indexOf("robocode") != 0)
                    && (getRootPackage() != null || className.equals(fullClassName))
                    &&!referencedClasses.containsKey(className))
                referencedClasses.put(className, "false");
        }

    }

    public void addResolvedClass(String className) {
        if (!referencedClasses.containsKey(className)) {
            try {

                // Try loading it externally
                robotClassLoader.loadExternalClass(className, true);

                referencedClasses.put(className, "true");
            }
            catch (ClassNotFoundException e) {
                System.out.println("FAILED");
                log(fullClassName + ": Cannot set " + className
                    + " to resolved, did not know it was referenced.");
            }

            return;
        }
        else {
            referencedClasses.put(className, "true");

            return;
        }
    }

    public String getFullClassName() {
        return fullClassName;
    }

    public Enumeration getReferencedClasses() {
        return referencedClasses.keys();
    }

    public Class getRobotClass() {
        return robotClass;
    }

    public RobocodeClassLoader getRobotClassLoader() {
        if (robotClassLoader == null)
            robotClassLoader =
                new RobocodeClassLoader(getClass().getClassLoader(), this);

        return robotClassLoader;
    }

    public RobotSpecification getRobotSpecification() {
        return robotSpecification;
    }

    public void loadUnresolvedClasses() throws ClassNotFoundException {
        for (Enumeration enum = referencedClasses.keys();
                enum.hasMoreElements(); ) {
            String s = (String) enum.nextElement();

            if (referencedClasses.get(s).equals("false")) {
                robotClassLoader.loadRobotClass(s, false);

                enum = referencedClasses.keys();
            }
        }

    }

    public void log(String s) {
        Utils.log(s);
    }

    public void log(Throwable e) {
        Utils.log(e);
    }

    public void setRobotClass(Class newRobotClass) {
        robotClass = newRobotClass;
    }

    public String toString() {
        return getRobotSpecification().getNameManager().getUniqueFullClassNameWithVersion();
    }

    public robocode.control.RobotSpecification getControlRobotSpecification() {
        return controlRobotSpecification;
    }

    public void setControlRobotSpecification(robocode.control.RobotSpecification controlRobotSpecification) {
        this.controlRobotSpecification = controlRobotSpecification;
    }

    public TeamPeer getTeamManager() {
        return teamManager;
    }

    public String getUid() {
        return uid;
    }

    public void setUid(String uid) {
        this.uid = uid;
    }

}

