/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package robocode.security;

import java.io.*;
import java.net.MalformedURLException;
import java.security.*;
import java.util.Hashtable;
import java.util.Vector;
import robocode.manager.NameManager;
import robocode.packager.ClassAnalyzer;
import robocode.peer.robot.RobotClassManager;
import robocode.repository.RobotSpecification;

public class RobocodeClassLoader extends ClassLoader {

    private Hashtable cachedClasses = null;
    private ClassLoader parent = null;
    private RobotSpecification robotSpecification = null;
    private RobotClassManager robotClassManager = null;
    private String rootPackageDirectory = null;
    private String rootDirectory = null;
    private String classDirectory = null;
    private String shortName = null;
    private Hashtable jarClasses = null;
    private ProtectionDomain protectionDomain = null;
    long uid1 = 0;
    long uid2 = 0;

    public RobocodeClassLoader(ClassLoader parent,
                               RobotClassManager robotClassManager) {
        super(parent);

        cachedClasses = new Hashtable();
        robotSpecification = null;
        this.robotClassManager = null;
        rootPackageDirectory = null;
        rootDirectory = null;
        classDirectory = null;
        shortName = null;
        jarClasses = new Hashtable();
        protectionDomain = null;
        uid1 = 0L;
        uid2 = 0L;
        this.parent = parent;
        this.robotClassManager = robotClassManager;
        robotSpecification = robotClassManager.getRobotSpecification();
    }

    public String getClassDirectory() {
        return classDirectory;
    }

    public InputStream getResourceAsStream(String resource) {
        log("Classloader:  getResourceAsStream: " + resource);

        return super.getResourceAsStream(resource);
    }

    public String getRootDirectory() {
        return rootDirectory;
    }

    public void finalize() {}

    public String getRootPackageDirectory() {
        return rootPackageDirectory;
    }

    public synchronized Class loadClass(String className, boolean resolve)
            throws ClassNotFoundException {
        if (className.indexOf(robotClassManager.getRootPackage() + ".") == 0)
            return loadRobotClass(className, false);

        try {
            Class c = super.loadClass(className, resolve);

            return c;
        }
        catch (ClassNotFoundException classnotfoundexception) {
            return loadRobotClass(className, false);
        }
    }

    public synchronized Class loadRobotClass(String name, boolean toplevel)
            throws ClassNotFoundException {
        if (cachedClasses.containsKey(name))
            return (Class) cachedClasses.get(name);

        Class c = null;
        File f = null;

        if (toplevel) {
            uid1 = 0L;
            uid2 = 0L;
        }

        if (!name.equals(robotClassManager.getFullClassName())
                && robotClassManager.getRootPackage() == null) {
            log(robotClassManager.getFullClassName()
                + " is not in a package, but is trying to reference class "
                + name);
            log("To do this in Robocode, you must put your robot into a package.");

            throw new ClassNotFoundException(robotClassManager.getFullClassName()
                                             + "is not in a package, but is trying to reference class "
                                             + name);
        }

        String filename = name.replace('.', File.separatorChar) + ".class";
        String classPath = robotSpecification.getRobotClassPath();

        if (classPath.indexOf(File.pathSeparator) >= 0)
            throw new ClassNotFoundException("A robot cannot have multiple directories or jars in it's classpath: "
                                             + name);

        f = new File(classPath + File.separator + filename);

        if (protectionDomain == null)
            try {
                Permissions p = new Permissions();

                protectionDomain =
                    new ProtectionDomain(new CodeSource(f.toURL(), null), p);
            }
            catch (MalformedURLException e) {
                throw new ClassNotFoundException("Unable to build protection domain.");
            }

        int size = (int) f.length();

        uid1 += size;

        byte buff[] = new byte[size];

        try {
            FileInputStream fis = new FileInputStream(f);
            DataInputStream dis = new DataInputStream(fis);

            dis.readFully(buff);
            dis.close();

            Vector v = ClassAnalyzer.getReferencedClasses(buff);

            robotClassManager.addReferencedClasses(v);

            uid1 += v.size();

            for (int i = 0; i < buff.length; i++)
                uid2 += buff[i];

            c = defineClass(name, buff, 0, buff.length, protectionDomain);

            robotClassManager.addResolvedClass(name);

            if (name.equals(robotClassManager.getFullClassName()))
                try {
                    if (robotClassManager.getRootPackage() == null) {
                        rootPackageDirectory = null;
                        classDirectory = null;
                        rootDirectory =
                            (new File(classPath)).getCanonicalPath();
                    }
                    else {
                        rootPackageDirectory =
                            (new File(classPath + File.separator
                                      + robotClassManager.getRootPackage()
                                      + File.separator)).getCanonicalPath();
                        classDirectory =
                            (new File(classPath + File.separator
                                      + robotClassManager.getClassNameManager().getFullPackage().replace('.', File.separatorChar)
                                      + File.separator)).getCanonicalPath();
                        rootDirectory =
                            (new File(classPath)).getCanonicalPath();
                    }
                }
                catch (Exception e) {
                    rootPackageDirectory =
                        (new File(classPath + File.separator
                                  + robotClassManager.getRootPackage()
                                  + File.separator)).getAbsolutePath();

                    log("Unexpected error:  Cannot build canonical path for "
                        + rootPackageDirectory);
                }

            if (toplevel) {
                robotClassManager.loadUnresolvedClasses();
                robotClassManager.setUid(uid1 + "" + uid2);
            }

            cachedClasses.put(name, c);

            return c;
        }
        catch (FileNotFoundException e) {

            // Try loading it externally (hack)
            c = loadExternalClass(name, true);

            robotClassManager.addReferencedClass(name, "true");
            robotClassManager.addResolvedClass(name);
            cachedClasses.put(name, c);

            return c;
        }
        catch (IOException e) {
            throw new ClassNotFoundException("Could not find: " + name + ": "
                                             + e);
        }
    }

    public synchronized Class loadExternalClass(String className, boolean resolve)
            throws ClassNotFoundException {

        try {
            System.out.print("* Loading external class: " + className + "... ");


            Class c = super.loadClass(className, resolve);

            System.out.println("OK");

            return c;
        }
        catch (ClassNotFoundException e) {

            // Just filter it out
            System.out.println("FAILED");
            throw e;
        }
    }


    private void log(String s) {
        System.out.println("SYSTEM: " + s);
    }

}

